'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.prop = prop;
exports.booleanClass = booleanClass;
exports.html = html;
exports.text = text;
exports.value = value;
exports.published = published;
exports.parts = parts;
exports.src = src;
exports.selectMultiple = selectMultiple;
function prop(property) {
  var negate = void 0;
  if (/^!/.test(property)) {
    negate = true;
    property = property.slice(1);
  }
  return function ($el, value) {
    $el = $($el);

    $el.prop(property, negate && !value || !negate && !!value);
  };
}

function booleanClass(setClass, unsetClass) {
  return function ($el, value) {
    $el = $($el);

    $el.toggleClass(setClass, !!value).toggleClass(unsetClass, !value);
  };
}

function html($el, value) {
  $el = $($el);

  $el.html(value);

  $el.trigger('change', { internal: true });
}

function text($el, value) {
  $el = $($el);

  $el.text(value);

  $el.trigger('change', { internal: true });
}

function value(el, value) {
  if (el.type === 'radio' || el.type === 'checkbox') {
    if (el.name && el.form) {
      el = el.form[el.name];
    }

    if ('length' in el) {
      $(el).prop('checked', false);

      if (value != null) {
        _.each(el, function (el) {
          if (_.isArray(value) && value.indexOf(el.value) > -1 || el.value === value) {
            el.checked = true;
          }
        });
      }
    } else {
      el.checked = !!value;
    }
  } else {
    if (value != null) value = value.toString();

    $(el).val(value);
  }

  $(el).trigger('change', { internal: true });
}

function published($el, value) {
  $el = $($el);

  $el = $el.closest('[data-published]');

  $el.attr('data-published', (!!value).toString());
}

function parts($el, value) {
  $el = $($el);

  if (!_.isDate(value)) value = new Date(value);

  var ref = value.toLocaleString('se-SV').split(' ');

  $el.find('[data-part]').toArray().forEach(function (part, index) {
    $(part).val(ref[index]);
  });
}

function src($el, value) {
  $el = $($el);

  $el.attr('src', value);
}

function selectMultiple($el, value) {
  $el = $($el);

  if (!value) {
    $el.val(null);
  } else {
    value.forEach(function (value) {
      $el.find('[value="' + value + '"]').prop('selected', true);
    });
  }
}