'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _ridge = require('./ridge');

var _ridge2 = _interopRequireDefault(_ridge);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createViews() {
  var self = this;

  return _.flatten(_.map(this.subviews, function (subview, name) {
    subview = parseSubview.call(self, subview);

    var Subview = subview.constructor;

    if (!Subview) return;

    if (!subview.multi) {
      // passing of falsy subview.el uses parent elements element, ie one
      // element gets more than one view
      subview.el = subview.el ? self.$(subview.el) : $(self.el);

      if (subview.el.length === 0) {
        return;
      }

      return self[name] = new Subview(subview);
    }

    return self[name] = _.map(self.$(subview.el), function (el) {
      subview.el = el;
      return new Subview(subview);
    });
  }));
}

/* Base view */

function parseSubview(subview) {
  var defaults = {
    collection: this.collection,
    model: this.model,
    state: this.state,
    data: this.data
  };

  if (_.isArray(subview)) {
    subview = _.extend({
      el: subview[0],
      constructor: subview[1]
    }, subview[2]);
  }

  subview.parent = this;
  // subview.name = name;

  if (!subview.prepareView) {
    _.defaults(subview, defaults);
  }

  return subview;
}

function getElements(view) {
  return _.mapValues(view.elements, function (selector) {
    selector = _.isString(selector) ? selector : selector.selector;

    return view.$(selector);
  });
}

var View = Backbone.View.extend({
  constructor: function constructor(options) {
    var _Backbone$View;

    _.extend(this, _.pick(options, 'template', 'parent', 'bindings', 'subviews', 'state', 'templateEngine', 'loadTemplate'));

    // we clone to prevent views referencing the same object
    this.data = options && options.data ? _.clone(options.data) : {};

    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    (_Backbone$View = Backbone.View).call.apply(_Backbone$View, [this, options].concat(args));

    if (this.rendering) return;

    this.rendered = this.el && (this.el.children.length > 0 || !!this.el.textContent.trim());

    if (!this.rendered) this.render();else this._attach();
  },
  render: function render() {
    _.result(this, 'unobserve');

    var data = _.extend({}, _ridge2.default.context, _.result(this.state, 'toJSON'), this.data, _.result(this.model, 'toJSON'));

    this.renderTemplate(data);

    return this;
  },


  // Render Dust template and update element, using the fx queue

  renderTemplate: function renderTemplate(data) {
    var result = this.template(data);

    this.updateElement(result);

    this._attach();

    this.rendered = true;

    return this;
  },
  updateElement: function updateElement(newEl) {
    var _this = this;

    if (typeof newEl === 'string') {
      newEl = $(newEl)[0];
    }

    this.$el.empty();
    this.$el.append(newEl.childNodes);

    _.forEach(newEl.attributes, function (node) {
      _this.$el.attr(node.nodeName, node.value);
    });

    // remove old subviews
    _.invokeMap(this.views, 'remove');
  },
  _attach: function _attach() {
    this._subviews = createViews.call(this);
    this.elements = getElements(this);
    if (this.attach) this.attach();
  },


  // animated enter
  enter: function enter() {
    var _$el;

    (_$el = this.$el).enter.apply(_$el, arguments);
    return this;
  },
  preventDefault: function preventDefault(e) {
    e.preventDefault();
  },
  toggle: function toggle(options) {
    this.$el.toggle(options);
    return this;
  },
  remove: function remove() {
    _.invokeMap(this._subviews, 'remove');

    this._removeElement();
    this.stopListening();
    return this;
  },


  // animated remove
  leave: function leave(options) {
    this.stopListening();

    var subViews = this.views;

    while (subViews && subViews.length > 0) {
      _.invokeMap(subViews, 'stopListening');

      subViews = _.compact(_.flatten(_.map(subViews, 'views')));
    }

    this.$el.leave(options, this.remove.bind(this));

    return this;
  }
});

View.prototype._remove = View.prototype.remove;
View.prototype._stopListening = View.prototype.stopListening;
View.prototype._render = View.prototype.render;

exports.default = View;
module.exports = exports['default'];