'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _model = require('../model');

var _model2 = _interopRequireDefault(_model);

var _validate = require('../mixins/validate');

var _validate2 = _interopRequireDefault(_validate);

var _message = require('./message');

var _message2 = _interopRequireDefault(_message);

var _formStyling = require('./form-styling');

var _formStyling2 = _interopRequireDefault(_formStyling);

var _ridge = require('../ridge');

var _ridge2 = _interopRequireDefault(_ridge);

var _view2 = require('../view');

var _view3 = _interopRequireDefault(_view2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var LoginFormView = _view3.default.extend();
var LoginFormModel = _model2.default.extend();

_.extend(LoginFormModel.prototype, _validate2.default, {
  urlRoot: '/auth/local',

  validation: {
    email: {
      email: true,
      required: true
    },
    password: {
      required: true
    }
  }
});

_.extend(LoginFormView.prototype, require('../mixins/observe'), {
  template: 'partials/login-form',

  events: {
    'click a.external': 'externalLogin',
    'click .close': 'close',
    'submit form': 'save'
  },

  subviews: {
    form: ['form', _formStyling2.default]
  },

  elements: {
    button: 'button',
    form: 'form'
  },

  initialize: function initialize(opts) {
    if (opts && opts.removeOnLogin) {
      this.listenTo(_ridge2.default, 'login', this.remove);
    }

    this.model = opts && opts.model || new _model2.default();

    this.bindings = _.mapValues(this.model.validation, function (value, key) {
      var binding = {};

      binding['[name="' + key + '"],[data-name="' + key + '"]'] = {
        both: 'value'
      };

      return binding;
    });
  },
  error: function error(model, xhr) {
    if (this.message) {
      this.message.leave({ animateHeight: true });
    }

    var resp = xhr.responseJSON;

    this.message = new _message2.default({
      message: {
        type: 'error',
        heading: resp.statusText,
        body: resp.message
      }
    }).enter(this.elements.form, { method: 'prepend', animateHeight: true });
  },
  save: function save(e) {
    e.preventDefault();

    if (this.model.isValid()) {
      $(document.body).addClass('progress');

      this.elements.button.prop('disabled', true);

      this.model.save(null, {
        error: this.error,
        success: this.success,
        complete: this.complete,
        context: this,
        validate: false
      });
    }
  },
  complete: function complete() {
    this.elements.button.prop('disabled', false);

    $(document.body).removeClass('progress');
  },
  success: function success(model, resp, options) {
    model.reset({ silent: true });

    _ridge2.default.login(resp, options.xhr.getResponseHeader('location'));
  },
  attach: function attach() {
    this.observe({ validate: true });
  },
  externalLogin: function externalLogin(e) {
    e.preventDefault();

    var _view = this;
    var newWindow = window.open($(e.currentTarget).attr('href') + '?loginWindow=true', 'name', 'height=600,width=450');

    if (window.focus) {
      newWindow.focus();
    }

    this.listenToOnce(window.broadcast, 'authenticate', function (err, user, newUser, redirect) {
      if (err) {
        if (_view.message) {
          _view.message.leave({ animateHeight: true });
        }

        _view.message = new _ridge2.default.views.Message({
          animateHeight: true,
          message: {
            type: 'error',
            heading: err.statusText,
            body: err.message
          }
        }).enter(e.currentTarget, 'before', true);
      } else {
        _ridge2.default.login(user, redirect);
      }
    });
  }
});

exports.default = LoginFormView;
module.exports = exports['default'];